#include <Keypad.h>
#include <Arduino.h>
#include <U8g2lib.h>

#ifdef U8X8_HAVE_HW_I2C
#include <Wire.h>
#endif

U8G2_SH1106_128X64_NONAME_1_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);

//Variables
const byte ROWS = 4; 
const byte COLS = 4; 
char char1 = '\0'; 
char char2 = '\0';
char char3 = '\0';
int num1 = -1;
int num2 = -1;
int num3 = -1;
int completeNum = 0; 
int maxHP = 0; 
int tempHP = 0;
int currentHealth = 0;
int heal = 0;
int damage = 0;
bool digitPress = false;

// Define states
enum State {
  MaxHP,
  TempHP,
  Heal,
  Damage,
};

//sets starting state
State currentState = MaxHP;

//set the keypad buttons to the right output
//A is for Max HP
//B is for Temporary HP
//C is for Healing
//D is for Taking Damage

//* is Deny number input
//# is Accept number input
char hexaKeys[ROWS][COLS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};

byte rowPins[ROWS] = {2, 3, 4, 5}; 
byte colPins[COLS] = {6, 7, 8, 9}; 

Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS); 

void setup(){
  Serial.begin(9600);
  u8g2.begin();

  //set the starting display
  u8g2.firstPage(); 
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(currentHealth);
      u8g2.setFont(u8g2_font_mystery_quest_28_tn);
      u8g2.setCursor(75, 25);
      u8g2.print("+ ");
      u8g2.setCursor(90, 25); 
      u8g2.print(tempHP);
      u8g2.setCursor(75, 50);
      u8g2.setFont(u8g2_font_streamline_interface_essential_setting_t);
      u8g2.drawStr(90, 60, "2");
    } while ( u8g2.nextPage() ); 
}
  
void loop(){
  //Detect current input
  char customKey = customKeypad.getKey();
  
  //cancelling number insert
  if (customKey == '*'){
    resetNums();
    digitPress = false; 
  }

  //Set State
  if (customKey == 'A' && currentState != MaxHP){
    currentState = MaxHP;
    currentHealth = maxHP;
    Serial.println("MaxHP");
    resetNums();
  }

  if (customKey == 'B' && currentState != TempHP){
    currentState = TempHP;
    Serial.println("TempHP");
    resetNums();
  }

  if (customKey == 'C' && currentState != Heal){
    currentState = Heal;
    Serial.println("Heal");
    resetNums();
  }

  if (customKey == 'D' && currentState != Damage){
    currentState = Damage;
    Serial.println("Damage");
    resetNums();
  }
  
  //Calculate number display
  if (isdigit(customKey)){
    digitPress = true;
    
    if (customKey != '\0' && char1 == '\0' && char1 != customKey){
    Serial.println(customKey);
    char1 = customKey;
    num1 = char1 - '0';

  } else if (customKey != '\0' && char2 == '\0' && char2 != customKey){
    Serial.println(customKey);
    char2 = customKey;
    num1 = num1 * 10;
    num2 = char2 - '0';

  } else if (customKey != '\0' && char3 == '\0' && char3 != customKey){
    Serial.println(customKey);
    char3 = customKey;
    num1 = num1 * 10;
    num2 = num2 * 10;
    num3 = char3 - '0';
    }
  }

  //set minimum number display to 0
  if (num1 < 0){
    num1 = 0;
  }

  if (num2 < 0){
    num2 = 0;
  }

  if (num3 < 0){
    num3 = 0;
  }

  //apply effect per state if Accept is pressed
  if (customKey == '#' && customKey != '\0' && !isDigit(customKey)){
    switch (currentState) {
      //set Max HP to number input
    case MaxHP:
      maxHP = num1 + num2 + num3;
      
      if (maxHP > 999){
        maxHP = 999; 
      }

      currentHealth = maxHP;
      break;
      
      //set Temporary HP to number input
    case TempHP:
      tempHP = num1 + num2 + num3;
      break;
      
      //add number input to currentHealth (Heal for number input)
    case Heal:
      heal = num1 + num2 + num3; 
      currentHealth += heal;
      break;

      //subtract number input from temporary HP and then currentHealth (take damage equal to number input)
    case Damage:
      damage = num1 + num2 + num3; 
      if (tempHP >0){
        tempHP -= damage;
        if (tempHP <0){
          damage = -tempHP;
        }
      }
      currentHealth -= damage; 
      break;
    }
    
    //set minimum currentHealth to 0
    if (currentHealth <0){
      currentHealth = 0;
    }

    //set minimum temporary HP to 0
    if (tempHP <0){
      tempHP = 0;
    }

    resetNums();
    digitPress = false;
  } 
  
  if (currentHealth > maxHP){
    currentHealth = maxHP;
  }

//set display to fit the current state, number input and/or health when something else than a number is pressed
  if (digitPress == false && customKey != '\0') {

    switch (currentState) {
    case MaxHP:

    u8g2.firstPage();
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(currentHealth);
      u8g2.setFont(u8g2_font_mystery_quest_28_tn);
      u8g2.setCursor(75, 25);
      u8g2.print("+ ");
      u8g2.setCursor(90, 25); 
      u8g2.print(tempHP);
      u8g2.setCursor(75, 50);
      u8g2.setFont(u8g2_font_streamline_interface_essential_setting_t);
      u8g2.drawStr(90, 60, "2");
    } while ( u8g2.nextPage() ); 
      break;
      
    case TempHP:

    u8g2.firstPage(); 
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(currentHealth);
      u8g2.setFont(u8g2_font_mystery_quest_28_tn);
      u8g2.setCursor(75, 25);
      u8g2.print("+ ");
      u8g2.setCursor(90, 25); 
      u8g2.print(tempHP);
      u8g2.setCursor(75, 50);
      u8g2.setFont(u8g2_font_streamline_hand_signs_t);
      u8g2.drawStr(90, 60, "5");
    } while ( u8g2.nextPage() ); 
      break;
      
    case Heal:

    u8g2.firstPage(); 
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(currentHealth);
      u8g2.setFont(u8g2_font_mystery_quest_28_tn);
      u8g2.setCursor(75, 25);
      u8g2.print("+ ");
      u8g2.setCursor(90, 25); 
      u8g2.print(tempHP);
      u8g2.setCursor(75, 50);
      u8g2.setFont(u8g2_font_streamline_health_beauty_t);
      u8g2.drawStr(90, 60, "J");
    } while ( u8g2.nextPage() ); 
      break;
    
    case Damage:

      u8g2.firstPage(); 
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(currentHealth);
      u8g2.setFont(u8g2_font_mystery_quest_28_tn);
      u8g2.setCursor(75, 25);
      u8g2.print("+ ");
      u8g2.setCursor(90, 25); 
      u8g2.print(tempHP);
      u8g2.setCursor(75, 50);
      u8g2.setFont(u8g2_font_streamline_interface_essential_setting_t);
      u8g2.drawStr(90, 60, "1");
    } while ( u8g2.nextPage() );   
      break;
  }
    
    resetStates();

    //set display to fit the current state, number input and/or health when a number is pressed
  } else if(customKey != '\0') {
    completeNum = num1 + num2 + num3;

    switch (currentState) {
    case MaxHP:

    u8g2.firstPage();
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(completeNum);
      u8g2.setCursor(90, 60);
      u8g2.setFont(u8g2_font_streamline_interface_essential_setting_t);
      u8g2.print(2);
    } while ( u8g2.nextPage() );
      break;
      
    case TempHP:

    u8g2.firstPage();
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(completeNum);
      u8g2.setCursor(90, 60);
      u8g2.setFont(u8g2_font_streamline_hand_signs_t);
      u8g2.print(5);
    } while ( u8g2.nextPage() );
      break;
      
    case Heal:

    u8g2.firstPage(); 
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(completeNum);
      u8g2.setCursor(75, 50);
      u8g2.setFont(u8g2_font_streamline_health_beauty_t);
      u8g2.drawStr(90, 60, "J");
    } while ( u8g2.nextPage() );
      break;
    
    case Damage:

      u8g2.firstPage();
    do {
      u8g2.setFont(u8g2_font_mystery_quest_48_tn);
      u8g2.setCursor(0, 50);
      u8g2.print(completeNum);
      u8g2.setCursor(75, 50);
      u8g2.setFont(u8g2_font_streamline_interface_essential_setting_t);
      u8g2.drawStr(90, 60, "1");
    } while ( u8g2.nextPage() );    
      break;
  }
  } 
}

//resets the variables in states to 0 (heal and damage)
void resetStates(){
  heal = 0;
  damage = 0;
}

//resets the inputs to Null
void resetNums(){
  char1 = '\0';
    char2 = '\0';
    char3 = '\0';
    num1 = -1;
    num2 = -1;
    num3 = -1;
}
